<?php

namespace Drupal\Tests\acquia_contenthub\Functional;

use Drupal\Component\Serialization\Json;
use Drupal\field_permissions\Plugin\FieldPermissionTypeInterface;

/**
 * Test that Acquia Content Hub respects Field Permissions.
 *
 * @group acquia_contenthub_no_test
 * @group orca_ignore
 */
class FieldPermissionsNodeTest extends ContentHubTestBase {

  /**
   * The permissions of the admin user.
   *
   * @var string[]
   */
  protected $adminUserPermissions = [
    'bypass node access',
    'administer acquia content hub',
    'administer content types',
    'access administration pages',
    'administer node fields',
    'administer field permissions',
    'access private fields',
  ];

  /**
   * Admin role.
   *
   * @var string
   */
  protected $adminRole;

  /**
   * Modules to enable for this test.
   *
   * @var string[]
   */
  public static $modules = [
    'node',
    'acquia_contenthub',
    'field',
    'field_ui',
    'user',
    'field_permissions',
  ];

  /**
   * {@inheritdoc}
   */
  public function setUp(): void {
    $this->markTestSkipped('Skipped until LCH-5683 is resolved.');
    parent::setUp();
  }

  /**
   * Configure content hub node form.
   */
  public function testFieldPermissions() {
    $this->adminRole = $this->createAdminRole('administrator', 'Administrator');
    $this->drupalLogin($this->adminUser);

    // Create Article Node.
    $entity = $this->drupalCreateNode([
      'type' => 'article',
      'title' => 'Test Title',
      'body'      => [[
        'value' => 'Test Body',
        'format' => filter_default_format(),
      ],
      ],
    ]);

    $this->configureContentHubContentTypes('node', ['article']);

    // Rendering the CDF as anonymous, we can see the "Body" Field.
    $output = $this->drupalGetCdf('acquia-contenthub-cdf/' . $entity->getEntityTypeId() . '/' . $entity->id(), [
      'query' => [
        'include_references' => 'true',
      ],
    ]);
    $this->assertSession()->statusCodeEquals(200);
    $this->assertEquals($entity->uuid(), $output['entities']['0']['uuid']);
    $this->assertEquals('Test Title', $output['entities']['0']['attributes']['title']['value']['en']);

    // Obtaining body.
    $json_body = $output['entities']['0']['attributes']['body']['value']['en'][0];
    $body = Json::decode($json_body);
    $this->assertEquals('Test Body', $body['value']);

    // Now modify article "Body" Field to have private access (only admin and
    // owner can view/edit the field).
    $this->drupalGet('admin/structure/types/manage/article/fields/node.article.body');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('Field visibility and permissions');

    $perm = FieldPermissionTypeInterface::ACCESS_PRIVATE;
    $edit = ['type' => $perm];
    $this->drupalGet(NULL);
    $this->submitForm($edit, $this->t('Save settings'));
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('Saved Body configuration');

    // Check field permissions settings for Body.
    $this->drupalGet('admin/structure/types/manage/article/fields/node.article.body');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->checkboxChecked('edit-type-private');

    // Clear cache because configuration was changed.
    drupal_flush_all_caches();

    // Rendering CDF as anonymous, we cannot see the "Body" field.
    $output = $this->drupalGetCdf('acquia-contenthub-cdf/' . $entity->getEntityTypeId() . '/' . $entity->id(), [
      'query' => [
        'include_references' => 'true',
      ],
    ]);
    $this->assertSession()->statusCodeEquals(200);
    $this->assertEquals($entity->uuid(), $output['entities']['0']['uuid']);
    $this->assertEquals('Test Title', $output['entities']['0']['attributes']['title']['value']['en']);
    $this->assertTrue(empty($output['entities']['0']['attributes']['body']), 'Body Field not present in CDF.');

    $this->setRoleFor($this->adminRole);

    // Rendering the CDF using the admin role, we can see the "Body" Field.
    $output = $this->drupalGetCdf('acquia-contenthub-cdf/' . $entity->getEntityTypeId() . '/' . $entity->id(), [
      'query' => [
        'include_references' => 'true',
      ],
    ]);
    $this->assertSession()->statusCodeEquals(200);
    $this->assertEquals($entity->uuid(), $output['entities']['0']['uuid']);
    $this->assertEquals('Test Title', $output['entities']['0']['attributes']['title']['value']['en']);

    // Obtaining body.
    $json_body = $output['entities']['0']['attributes']['body']['value']['en'][0];
    $body = Json::decode($json_body);
    $this->assertEquals('Test Body', $body['value']);
  }

}
